#include "sh.h"
#include <sys/stat.h>

/*
 * Simplify pathnames containing "." and ".." entries.
 * ie, simplify_path("/a/b/c/./../d/..") returns "/a/b"
 */
void
simplify_path(char *path)
{
	char	*cur;
	char	*t;
	int	isrooted;
	char	*very_start = path;
	char	*start;

	if (!*path)
		return;

	if ((isrooted = path[0] == '/'))
		very_start++;

	/* Before			After
	 *  /foo/			/foo
	 *  /foo/../../bar		/bar
	 *  /foo/./blah/..		/foo
	 *  .				.
	 *  ..				..
	 *  ./foo			foo
	 *  foo/../../../bar		../../bar
	 */

	for (cur = t = start = very_start; ; ) {
		/* treat multiple '/'s as one '/' */
		while (*t == '/')
			t++;

		if (*t == '\0') {
			if (cur == path)
				/* convert empty path to dot */
				*cur++ = '.';
			*cur = '\0';
			break;
		}

		if (t[0] == '.') {
			if (!t[1] || t[1] == '/') {
				t += 1;
				continue;
			} else if (t[1] == '.' && (!t[2] || t[2] == '/')) {
				if (!isrooted && cur == start) {
					if (cur != very_start)
						*cur++ = '/';
					*cur++ = '.';
					*cur++ = '.';
					start = cur;
				} else if (cur != start)
					while (--cur > start && *cur != '/')
						;
				t += 2;
				continue;
			}
		}

		if (cur != very_start)
			*cur++ = '/';

		/* find/copy next component of pathname */
		while (*t && *t != '/')
			*cur++ = *t++;
	}
}

/* Copy path string to current_wd and $PWD */
void
set_current_wd(const char *path)
{
	size_t len = strlen(path) + 1;

	current_wd = aresize(current_wd, len, APERM);
	memcpy(current_wd, path, len);
	simplify_path(current_wd);
	/* Ignore failure (happens if readonly or integer) */
	setstr(global("PWD"), current_wd, KSH_RETURN_ERROR);
}

/* Figure out the current working directory from environment or kernel */
void
import_current_wd(void)
{
	char *p = str_val(global("PWD"));
	struct stat a, b;

	/* Use existing $PWD if valid (e.g. to avoid resolving symlinks) */
	if (p[0] == '/' && !stat(p, &a)
			&& !stat(".", &b)
			&& a.st_dev == b.st_dev
			&& a.st_ino == b.st_ino) {
		set_current_wd(p);
	} else if ((p = ksh_get_wd())) {
		set_current_wd(p);
		afree(p, ATEMP);
	} else
		set_current_wd("");
}
