/* Portable signal information database
 * J. Welsh, May 2019
 * Public domain
 */

/* to get NSIG for ksh hack */
#define _BSD_SOURCE
#define _DEFAULT_SOURCE

#include <signal.h>
#include <string.h>

typedef struct {
	int num;
	const char *name;
	const char *msg;
} row;

#define SIG(name,msg) {SIG##name,#name,msg}

static const row sigs[] = {
	/* ksh hacks */
	{0,"EXIT","Null signal/ksh exit trap"},
	{NSIG,"ERR","Error handler"},

	SIG(ABRT,"Aborted"),
	SIG(ALRM,"Alarm timer expired"),
	SIG(BUS,"Bus error"),
	SIG(CHLD,"Child status changed"),
	SIG(CONT,"Continued"),
#ifdef SIGEMT
	SIG(EMT,"Emulator trap"),
#endif
	SIG(FPE,"Arithmetic exception"),
	SIG(HUP,"Hangup"),
	SIG(ILL,"Illegal instruction"),
#ifdef SIGINFO
	SIG(INFO,"Information request"),
#endif
	SIG(INT,"Interrupt (terminal)"),
	SIG(IO,"I/O possible"),
	SIG(KILL,"Killed"),
	SIG(PIPE,"Write to broken pipe"),
	SIG(POLL,"Pollable event"),
	SIG(PROF,"Profiling timer expired"),
	SIG(PWR,"Power failure"),
	SIG(QUIT,"Quit (terminal)"),
	SIG(SEGV,"Segmentation fault"),
	SIG(STOP,"Stopped (signal)"),
	SIG(SYS,"Bad system call"),
	SIG(TERM,"Terminated"),
	SIG(TRAP,"Trace/breakpoint trap"),
	SIG(TSTP,"Stopped"),
	SIG(TTIN,"Stopped (tty input)"),
	SIG(TTOU,"Stopped (tty output)"),
	SIG(URG,"Urgent I/O condition"),
	SIG(USR1,"User-defined signal 1"),
	SIG(USR2,"User-defined signal 2"),
	SIG(VTALRM,"Virtual timer expired"),
	SIG(WINCH,"Window size changed"),
	SIG(XCPU,"CPU time limit exceeded"),
	SIG(XFSZ,"File size limit exceeded"),
};
#define ROWS (sizeof sigs/sizeof *sigs)

int sig_number(const char *name) {
	size_t i;
	for (i=0; i<ROWS; ++i)
		if (strcmp(sigs[i].name,name) == 0) return sigs[i].num;
	return -1;
}

const char *sig_name(int num) {
	size_t i;
	for (i=0; i<ROWS; ++i)
		if (sigs[i].num == num) return sigs[i].name;
	return 0;
}

const char *sig_message(int num) {
	size_t i;
	for (i=0; i<ROWS; ++i)
		if (sigs[i].num == num) return sigs[i].msg;
	return "Unknown signal";
}

size_t sig_name_maxlen(void) {
	static size_t max;
	if (!max) {
		size_t i;
		for (i=0; i<ROWS; ++i) {
			size_t l=strlen(sigs[i].name);
			if (l>max) max=l;
		}
	}
	return max;
}

size_t sig_message_maxlen(void) {
	static size_t max;
	if (!max) {
		size_t i;
		for (i=0; i<ROWS; ++i) {
			size_t l=strlen(sigs[i].msg);
			if (l>max) max=l;
		}
	}
	return max;
}
