P=clisp
V=2.49
MAJOR=0
MINOR=0

# Build requires: autoconf (tested 2.69), automake (tested 1.15), readline, ncurses
# Links with ncurses if found
# Recommended: cl-hyperspec

S="
$P-$V.tar.bz2 http://ftp.gnu.org/gnu/$P/release/$V/$P-$V.tar.bz2
libsigsegv-2.9.tar.gz
libsigsegv-2.9-kill-libtool.patch
clisp-2.49-no-page.h.patch
clisp-2.49-autoconf.patch
version.sh
"

SHA512="
eef66fc85199a2c283b616db61bf67ff103eeb0f19fa907da48994dc790b6f5f8d0c74fb3bd723c6b827c0ff3cfd89fa6ba67934fc669ed5d5249044b5140d81  clisp-2.49.tar.bz2
c059fa89eb3e85bad22d045de074bfc88fae42454821e10905ffd67209b6e6ea44eeab120b869696f77b816aacb9a5af3e4b51c640512e876df1b6da6f296bd4  libffcall-1.13.tar.gz
8088bb1aa39fb5fa77de7fd4abaf05d2d320b65a22087c71525e6a661fe8233bc14c5d2203fb8f3957a2f410a6953a714c446667fc4615bcb7a2d17bc0c3c91e  libsigsegv-2.9.tar.gz
"

build () {
	build=$PWD
	tar xzf libsigsegv-2.9.tar.gz
	tar xjf $P-$V.tar.bz2

	cd libsigsegv-2.9
	patch -p1 <../libsigsegv-2.9-kill-libtool.patch
	rm -f m4/libtool.m4 m4/lt* build-aux/* Makefile.in */Makefile.in aclocal.m4 config.h.in configure
	aclocal -I m4
	autoheader
	automake -a
	autoconf
	CFLAGS=-O2 ./configure --prefix=$build
	make -j$JOBS
	make check
	make install

	cd ../$P-$V
	cp ../version.sh .
	patch -p1 <../clisp-2.49-no-page.h.patch
	patch -p1 <../clisp-2.49-autoconf.patch
	# Add rawsock and remove i18n from the base linking set
	base_mods="syscalls regexp rawsock"
	mkdir saved
	(cd modules; mv $base_mods ../saved)
	rm -r modules; mv saved modules
	(cd modules; rm */gllib/Makefile.in */config.h.in */configure)
	(cd src
	    (cd m4; rm libtool.m4 lt* filecharset.m4 eloop.m4)
	    (cd glm4; rm gettext.m4 iconv.m4)
	    rm aclocal.m4 build-aux/ltmain.sh gllib/Makefile.in config.h.in configure
	    rm -r po
	    grep -v AC_CONFIG_MACRO_DIR configure.in >tmp; mv tmp configure.in
	    sed s,http://clisp.cons.org/impnotes/,file:///gales/pkg/$Q/doc/doc/impnotes.html, cfgunix.lisp >tmp; mv tmp cfgunix.lisp)
	# I tried to bypass this but the reconf process is insanely complex
	make -f Makefile.devel configures MODULES="$base_mods" GNULIB_USERS="src modules/regexp modules/syscalls"
	# CFLAGS= is their recommended way to disable debug syms; don't use strip.
	# They still add assorted optimization flags.
	./configure \
		CFLAGS= \
		CPPFLAGS="-I$build/include -I/gales/pkg/readline/include" \
		LDFLAGS="-L$build/lib -L/gales/pkg/readline/lib -L/gales/pkg/ncurses/lib" \
		--prefix=/gales/pkg/$Q \
		--datarootdir=/gales/pkg/$Q \
		--docdir=/gales/pkg/$Q/doc \
		--hyperspec=file:///gales/pkg/cl-hyperspec \
		--with-readline \
		--without-ffcall \
		--without-unicode \
		--without-dynamic-modules
	cd src
	ulimit -s 16384
	make install BASE_MODULES="$base_mods" DESTDIR=$build/root

	cd $build/root/gales/pkg/$Q
	(cd lib/$P-$V
	    # Pointless duplicate of the whole shebang
	    rm -r full
	    # So much for --without-unicode
	    rm data/UnicodeDataFull.txt)
	# More pointless duplicates
	(cd doc; rm clisp.html clisp-link.html doc/clisp.1 doc/clisp-link.1)
}

package () {
	cd root/gales/pkg
	a gales/pkg/$Q
	l gales/pkg/$P $Q
	l gales/command/clisp ../pkg/$P/bin/clisp
	l gales/command/clisp-link ../pkg/$P/bin/clisp-link
	l bin/clisp /gales/pkg/$P/bin/clisp #! portability
	l gales/doc/$P ../pkg/$P/doc
	l gales/man/$P ../pkg/$P/man
}
