# gbuild-run.sh: Gales Linux package build runner
# This is run in a clean environment by the gbuild wrapper.
# J. Welsh, 2017

set -e

. ./build.sh
COMPRESS="gzip -n"
PKG_EXT=.sh.gz
GBUILD_CONF=${GBUILD_CONF:-/etc/gbuild.conf}
[ -f "$GBUILD_CONF" ] && . "$GBUILD_CONF"
pkg=$P-$MAJOR.$MINOR-$V$PKG_EXT

die () {
	printf %s\\n "${1:-FAILED: $?}" >&2
	exit 1
}

verify_sha512 () {
	while read -r sum name; do
		if [ "$1" = "$name" ]; then
			printf '(SHA512) '
			printf '%s  %s\n' "$sum" "$name" | sha512sum -c || die
			return
		fi
	done <<-.
	$SHA512
	.
	die "Missing checksum for $1"
}

echo ">> Preparing build dir"
[ -f build/.gbuild_temp ] && rm -rf build
mkdir build
touch build/.gbuild_temp
cd build

echo ">> Preparing sources"
ln -s ../build.sh .
while read -r f url; do
	[ -z "$f" ] && continue
	if [ -f "../$f" ]; then
		printf %s\\n "$f"
		ln -s "../$f" .
	elif [ -f "/gales/dist/src/$f" ]; then
		ln -s "/gales/dist/src/$f" .
		verify_sha512 "$f"
	else die "Missing source $f"
	fi
done <<.
$S
.

sub="/bin/sh $PWD/../gbuild-sub.sh"

echo ">> Building"
$sub build || die

echo ">> Running tests"
$sub check || die

printf '>> Writing %s\n' "$pkg"
rm -f "../$pkg"
(set +e; $sub package; echo $? >.gbuild_status) | $COMPRESS >../$pkg
status=`cat .gbuild_status`
if [ $status -ne 0 ]; then
	rm -f "../$pkg"
	die "FAILED: $status"
fi

echo ">> Removing build dir"
cd ..
rm -rf build
