P=glib
V=2.25.14
MAJOR=0
MINOR=0

# Build requires: zlib

# Includes bundled:
# - pcre (--disable-regex doesn't actually work, as glib.h still includes gregex.h)

# Omitted optional dependencies:
# - GNU Gettext
# - FAM
# - xsltproc (to rebuild man pages)
# - gtk-doc (to rebuild html docs)
# - libdbus1 (for GDBus test cases)

# 2.25.15 added a requirement of the recent 'gz_header' feature in zlib.
# 2.29.4 added gdbus-codegen and with it a hard requirement of python and libffi.
# 2.32 included a buggy rework of gthreads.
# 2.58 dropped the autoconf build system in favor of 'meson', thus requiring python, which at some point morphed into python3.
# Somewhere in here a libelf option was added.

S="
$P-$V.tar.bz2
glib-2.25.14-no-test.mo.patch
stub-msgfmt.sh
"

SHA512="
c9d371c6ca38f0a54f17402927926404c20c6dd331b09324cdc893c089ec99919d57da324f3276e90fd95b1cf917bfc5b6688d71b92653dcddf395dec22d9bdc  glib-2.25.14.tar.bz2
"

build () {
	root=$PWD/root
	tar xjf $P-$V.tar.bz2
	cd $P-$V
	patch -p1 <../glib-2.25.14-no-test.mo.patch
	# Workaround for GNU Gettext build dependency: just don't build the translations.
	mkdir tmp_bin
	install ../stub-msgfmt.sh tmp_bin/msgfmt
	export PATH=$PWD/tmp_bin:$PATH
	# They point you at a ZLIB_CFLAGS and ZLIB_LIBS, but the first doesn't propagate to where it needs to be, so we use the generic variables.
	CPPFLAGS="-I/gales/pkg/zlib/include -D_GNU_SOURCE" \
	LDFLAGS=-L/gales/pkg/zlib/lib \
	./configure --prefix=/gales/pkg/$Q \
		--datarootdir=/gales/pkg/$Q \
		--disable-shared \
		--enable-static
	# FIXME:
	#	checking for struct stat.st_mtim.tv_nsec... no
	# Why?
	#	conftest.c:111:20: error: storage size of 'ac_aggr' isn't known
	# Why not?
	#	| #ifdef HAVE_SYS_STAT_H
	#	| # include <sys/stat.h>
	#	| #endif
	# and HAVE_SYS_STAT_H is never set.
	# Why not? Perhaps because it's not included in AC_CHECK_HEADERS. But likewise in 2.48.2, which didn't have this problem.
	# We'll live without nanosecond-timestamps for now.
	make -j$JOBS
	make install DESTDIR=$root
	cd $root/gales/pkg/$Q/bin
	# Fatass binaries.
	strip gdbus gio-querymodules glib-compile-schemas glib-genmarshal gobject-query gsettings gtester
	# Python script where #! interpreter path gets misconfigured if python isn't present at build time.
	rm -f gtester-report
	# Developers are directed not to use these prefixes when including header files. So how's the compiler supposed to find them? By complicating all the build systems with pkg-config and its metadata files, because why do anything the simple and obvious way when you can add more layers of shit, right?
	cd ../include
	mv glib-2.0/* .
	mv gio-unix-2.0/gio/* gio/
	rmdir gio-unix-2.0/gio gio-unix-2.0 glib-2.0
	cd ../lib
	mv glib-2.0/include/glibconfig.h ../include/
	rmdir glib-2.0/include glib-2.0
	rm -f *.la
	rmdir locale/*/LC_MESSAGES locale/* locale
}

check () {
	cd $P-$V
	#make check
	echo 'skipping broken test suite'
	# Fails with:
	#  nm: '.libs/libglib-2.0.so': No such file
	# It'll probably also fail because of translations.
}

package () {
	cd $P-$V
	f gales/pkg/$Q/doc/AUTHORS
	f gales/pkg/$Q/doc/NEWS
	f gales/pkg/$Q/doc/README
	cd ../root/gales/pkg
	a gales/pkg/$Q
	l gales/pkg/$P $Q
	for f in gdbus gio-querymodules glib-compile-schemas glib-genmarshal glib-gettextize glib-mkenums gobject-query gsettings gtester ; do
		l gales/command/$f ../pkg/$P/bin/$f
	done
	for f in glib-2.0.m4 glib-gettext.m4 gsettings.m4 ; do
		l gales/aclocal/$f ../pkg/$P/aclocal/$f
	done
	l gales/doc/$P ../pkg/$P/gtk-doc
	l gales/man/$P ../pkg/$P/man
}
